==> plot_window-0.0.38.tar.gz: lib/point_list.dart <==
part of plot_window;

typedef num PointMapper(Point p);
typedef num NumReducer(num n1, num n2);

class PointList {

  static NumReducer _ADDER = (num n1, num n2)=>n1+n2;

  static num _ReducePoints(List<Point> points, PointMapper pm, [NumReducer nr = null]) {
    if (nr==null) nr = _ADDER;
    return points.map(pm).reduce(nr);
  }

  List<Point> points;

  num _average;
  num _variance;
  num _deviation;

  num __sumY;
  num get _sumY {
    if (__sumY==null)
      __sumY = _ReducePoints(_lrPoints, (Point p)=>p.y); //this.points.map((Point p)=>p.y).reduce((value, element)=>value+element);
    return __sumY;
  }
  
  List<Point> __lrPoints;
  List<Point> get _lrPoints {
    if (__lrPoints==null) 
      __lrPoints=this.points.skipWhile((Point p)=>p.x<linearRegressionStart).takeWhile((Point p)=>p.x<=linearRegressionEnd).toList();
    return __lrPoints;
  }

  num get average {
    if (_average==null)
      _average=_sumY/points.length;
    return _average;
  }
  
  num get variance {
    if (_variance==null)
      _variance = _ReducePoints(_lrPoints, (Point p)=>math.pow(p.y, 2))/points.length - math.pow(average, 2);
    return _variance;
  }

  num get deviation {
    if (_deviation==null)
      _deviation = math.pow(variance, 0.5);
    return _deviation;
  }

  void _clearLinearRegression() {
    __sumY=null;
    _average=null;
    _variance=null;
    _deviation=null;
    __lrPoints=null;
    _m=null;
    _c=null;
  }

  num _linearRegressionStart;
  num get linearRegressionStart => _linearRegressionStart;
  void set linearRegressionStart(num n) {
    _clearLinearRegression();
    _linearRegressionStart = n;
  }
  
  num _linearRegressionEnd;
  num get linearRegressionEnd => _linearRegressionEnd;
  void set linearRegressionEnd(num n) {
    _clearLinearRegression();
    _linearRegressionEnd = n;
  }
  
  num _m;
  num _c;

  String getLinearRegressionString() {
    if (c<0) return "${m}x - ${-c}";
    return "${m}x + $c";
  }
  
  num get m {
    if (_m==null) _setLinearRegression();
    return _m;
  }
  num get c {
    if (_c==null) _setLinearRegression();
    return _c;
  }
  
  PointList(this.points) {
    linearRegressionStart = this.points.first.x;
    linearRegressionEnd = this.points.last.x;
  }

  /*
    y = m*x + c
    zu minimieren ist das Quadrat der Abstaende:
    s = sum( (y_i - (m*x_i +c))^2) 
    ds/dm = 2 * sum ( (y_i - (m*x_i + c)) * -x_i = 0
    S (x*y - m*x^2 - c*x ) = 0
    m * S (x^2) + c * S (x) = S (x*y)
    und

    ds/dc = 2 * sum( y - (m*x + c)) = 0
    S ( y - m*x - c) = 0
    m * S (x) + c * S(1) = S(y)

    | S(x^2)  S(x) |   | m |   | S(x*y) |
    |              | x |   | = |        |
    | S(x)    S(1) |   | c |   | S(y)   |

    AX = B
    X = A^{-1}B
   */

  
  void _setLinearRegression() {
    if (_m==null || _c == null) {
      print(_lrPoints);
      num sumX2 = _ReducePoints(_lrPoints, (Point p)=>math.pow(p.x, 2));
      num sumX = _ReducePoints(_lrPoints, (Point p)=>p.x);
      num sum = this.points.length;
      num sumXY = _ReducePoints(_lrPoints, (Point p)=>p.x*p.y);
      num sumY = _sumY;
      num det = sum*sumX2 - sumX*sumX;
      _m = (sum * sumXY - sumX*sumY)/det;
      _c = (-sumX*sumXY + sumX2*sumY) / det;
    }
  }

  PointList smooth(PointSmoother smoother) {
    return new PointList(smoother.smooth(points));
  }

}

==> plot_window-0.0.46.tar.gz: lib/point_list.dart <==
part of plot_window;

typedef num PointMapper(Point p);
typedef num NumReducer(num n1, num n2);

class PointList {

  static NumReducer _ADDER = (num n1, num n2)=>n1+n2;

  static num _ReducePoints(List<Point> points, PointMapper pm, [NumReducer nr = null]) {
    if (nr==null) nr = _ADDER;
    if (points.length==0) return null;
    return points.map(pm).reduce(nr);
  }

  List<Point> points;

  
  bool _cropLinearRegression = true;
  void set cropLinearRegression(bool val) {
    _clearLinearRegression();
    this._cropLinearRegression = val;
  }
  bool get cropLinearRegression=>_cropLinearRegression;
  
  num _average;
  num _variance;
  num _deviation;

  num __sumY;
  num get _sumY {
    if (__sumY==null)
      __sumY = _ReducePoints(_lrPoints, (Point p)=>p.y); //this.points.map((Point p)=>p.y).reduce((value, element)=>value+element);
    return __sumY;
  }
  
  List<Point> __lrPoints;
  List<Point> get _lrPoints {
    if (__lrPoints==null) {
      if(cropLinearRegression)
        __lrPoints=this.points.skipWhile((Point p)=>p.x<linearRegressionStart).takeWhile((Point p)=>p.x<=linearRegressionEnd).toList();
      else
        __lrPoints = this.points;
    }
    return __lrPoints;
  }

  num get average {
    if (_average==null)
      _average=_sumY/points.length;
    return _average;
  }
  
  num get variance {
    if (_variance==null)
      _variance = _ReducePoints(_lrPoints, (Point p)=>math.pow(p.y, 2))/points.length - math.pow(average, 2);
    return _variance;
  }

  num get deviation {
    if (_deviation==null)
      _deviation = math.pow(variance, 0.5);
    return _deviation;
  }

  void _clearLinearRegression() {
    __sumY=null;
    _average=null;
    _variance=null;
    _deviation=null;
    __lrPoints=null;
    _m=null;
    _c=null;
  }

  num _linearRegressionStart;
  num get linearRegressionStart => _linearRegressionStart;
  void set linearRegressionStart(num n) {
    _clearLinearRegression();
    _linearRegressionStart = n;
  }
  
  num _linearRegressionEnd;
  num get linearRegressionEnd => _linearRegressionEnd;
  void set linearRegressionEnd(num n) {
    _clearLinearRegression();
    _linearRegressionEnd = n;
  }
  
  num _m;
  num _c;

  String getLinearRegressionString() {
    if (m==null || c==null || m.isNaN || c.isNaN) return "undefined";
    if (c<0) return "${m}x - ${-c}";
    return "${m}x + $c";
  }
  
  num get m {
    if (_m==null) _setLinearRegression();
    return _m;
  }
  num get c {
    if (_c==null) _setLinearRegression();
    return _c;
  }
  
  PointList(this.points) {
    linearRegressionStart = this.points.first.x;
    linearRegressionEnd = this.points.last.x;
  }

  /*
    y = m*x + c
    zu minimieren ist das Quadrat der Abstaende:
    s = sum( (y_i - (m*x_i +c))^2) 
    ds/dm = 2 * sum ( (y_i - (m*x_i + c)) * -x_i = 0
    S (x*y - m*x^2 - c*x ) = 0
    m * S (x^2) + c * S (x) = S (x*y)
    und

    ds/dc = 2 * sum( y - (m*x + c)) = 0
    S ( y - m*x - c) = 0
    m * S (x) + c * S(1) = S(y)

    | S(x^2)  S(x) |   | m |   | S(x*y) |
    |              | x |   | = |        |
    | S(x)    S(1) |   | c |   | S(y)   |

    AX = B
    X = A^{-1}B
   */

  
  void _setLinearRegression() {
    if (_m==null || _c == null) {
      if (_lrPoints.length==0) return;
      //      print(_lrPoints);
      num sumX2 = _ReducePoints(_lrPoints, (Point p)=>math.pow(p.x, 2));
      num sumX = _ReducePoints(_lrPoints, (Point p)=>p.x);
      num sum = _lrPoints.length;
      num sumXY = _ReducePoints(_lrPoints, (Point p)=>p.x*p.y);
      num sumY = _sumY;
      num det = sum*sumX2 - sumX*sumX;
      //      print("$sumX2, $sumX, $sum, $sumXY, $sumY, $det");
      _m = (sum * sumXY - sumX*sumY)/det;
      _c = (-sumX*sumXY + sumX2*sumY) / det;
    }
  }

  PointList smooth(PointSmoother smoother) {
    return new PointList(smoother.smooth(points));
  }

}
